@@ -3,6 +3,8 @@ from TCLIService.ttypes import TOpenSessionReq, TGetTablesReq, TFetchResultsReq,
   TExecuteStatementReq, TGetOperationStatusReq, TFetchOrientation, TCloseOperationReq, \
   TCloseSessionReq, TGetSchemasReq, TGetLogReq, TCancelOperationReq
 
+from error import Pyhs2Exception
+
 def get_type(typeDesc):
     for ttype in typeDesc.types:
         if ttype.primitiveEntry is not None:
@@ -37,23 +39,33 @@ def get_value(colValue):
 class Cursor(object):
     session = None
     client = None
-    operationHandle =  None
-    def __init__(self, _client, session):
-        res = _client.OpenSession(TOpenSessionReq())
-        self.session = res.sessionHandle
+    operationHandle = None
+
+    def __init__(self, _client, sessionHandle):
+        self.session = sessionHandle
         self.client = _client
 
     def execute(self, hql):
         query = TExecuteStatementReq(self.session, statement=hql, confOverlay={})
         response = self.client.ExecuteStatement(query)
         self.operationHandle = response.operationHandle
-
+        if response.status.errorCode is not None:
+            raise Pyhs2Exception(response.status.errorCode, response.status.errorMessage)
+        
     def fetch(self):
         rows = []
-        fetchReq = TFetchResultsReq(operationHandle=self.operationHandle, orientation=TFetchOrientation.FETCH_NEXT, maxRows=100);
+        fetchReq = TFetchResultsReq(operationHandle=self.operationHandle,
+                                    orientation=TFetchOrientation.FETCH_NEXT,
+                                    maxRows=100)
         self._fetch(rows, fetchReq)
         return rows
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, _exc_type, _exc_value, _traceback):
+        self.close()
+
     def _fetch(self, rows, fetchReq):
         resultsRes = self.client.FetchResults(fetchReq)
         for row in resultsRes.results.rows:
@@ -67,5 +79,6 @@ class Cursor(object):
             return rows
 
     def close(self):
-        req = TCloseOperationReq(operationHandle=self.operationHandle)
-        self.client.CloseOperation(req) 
+        if self.operationHandle is not None:
+            req = TCloseOperationReq(operationHandle=self.operationHandle)
+            self.client.CloseOperation(req) 
